#!/usr/local/bin/perl -w
#
#  base_R.pl
#  v1.0
#
#  Execute R-script from BASE plugin
#  Let R handle BASEfile parsing
#  Created 07/08/2003 by ola@crossoverbio.com
#  for The Linnaeus Centre for Bioinformatics, www.lcb.uu.se
#
#  pl-script based on work by dpark@molbio.mgh.harvard.edu
#

use strict;

my $R_plug_dir = "/usr/local/base/plugins/R";
my %parlist;

sub require_baseargs;
sub read_basefile;

MAIN:
{
    warn "Plug-in starting\nReading BASEfile...\n";
    read_basefile;
    require_baseargs("fname");
    warn "Running plug-in: ".$parlist{fname}."\n";
    system("R --no-save < $R_plug_dir/$parlist{fname} > r-debug.txt") == 0
	or die "R script failed. Please see r-debug.txt for error information.\n Error($!)";
    warn "Plug-in completed\n";
}

sub require_baseargs
{
    foreach (@_) {
        die "\'$_\' variable not specified!" if not defined $parlist{$_};
    }
}

sub read_basefile
{
    my $currentsection = "start";
    my $header = 1;
    
    while (<STDIN>) {
	chomp;

	# empty lines denote the end of a section
	# remove leading (optional) '#' from headers
	# % denotes the start of the data within a section
	if(!length) { $currentsection="inter-section"; next; }
	if($header and /^\#/) { s/^\#//; }
	if(/^%$/)   { $header=0; next; }	

	# The beginning "BASEfile" section seems optional, including its
	#  "%" line and empty line that all other sections have.
	$currentsection = "inter-section"  if(/section\s+\w+/ and ($currentsection eq "BASEfile"));

	# Begin processing line depending on which section we're in.
	if($currentsection eq "start") {               # first line of file

	    /BASEfile/         or die '"BASEfile" not found on first line';
	    $currentsection = "BASEfile"; 
	}
	elsif($currentsection eq "BASEfile") {

	    /^(\S+)\t(\S+)/    or die "plugin parameters- not BASEfile format";
	    $parlist{$1} = $2;
	}
	elsif($currentsection eq "inter-section") {    # space btwn sections

	    /section\s+(\w+)/  or die "new section does not begin with section name on line $.!";
	    $currentsection = $1;
	    $header = 1;
	}
    }
}
## end of script
