#====================================
# Filename: parse.basefile.R
# Date: 2003/08/14
# Updated: 2003/09/22
# v: 0.92
#
# - Parse BASEfile, read plugins data from BASE, Transfer to Bioconductor(LIMMA) format
# - Writes transformations back to BASE after e.g. normalizations
#
# Functions:
#
# 1) read.basefile
# 2) read.basefile.short
# 3) read.basefile.short.names
# 4) write.transformation
# 5) write.reporterlist (NA yet)
#
# By: Ola Spjuth, ola@crossoverbio.com
# for The Linnaeus Centre for Bioinformatics, www.lcb.uu.se
#====================================

#====================================
# Function: write.reporterlist
#
# Write a Reporterlist as BASEfile
#====================================

write.reporterlist<-function(fname,RG)
{

#******
# TODO
#******

}

#====================================
# Function: write.transformation
#
# Write a 2ch BASEfile to file as trafo
#====================================

write.transformation<-function(fname,RG)
{

	#Handle header
	for(i in 1:length(RG$trafo)){
		if (unlist(strsplit(RG$trafo[i],"\t"))[1]=="assayFields")
			RG$trafo[i]="assayFields\tintensity1\tintensity2"
	}
	writeLines(text=RG$trafo, con=fname, sep="\n")
	
	#Handle data
	out<-matrix(NA,dim(RG$R)[1],dim(RG$R)[2]+dim(RG$G)[2]+3)	#+3 bc of pos,rep,repID 
	
	#Set three first lines
	out[,1]<-RG$pars[,1]
	out[,2]<-RG$pars[,2]
	out[,3]<-as.character(RG$pars[,3])
	
	nextl<-4
	for(ii in 1:dim(RG$R)[2])
	{
		out[,nextl]<-RG$R[,ii]
		nextl<-nextl+1
		out[,nextl]<-RG$G[,ii]
		nextl<-nextl+1
	}

	write.table(out,file=fname,quote=FALSE,col.names=FALSE,row.names=FALSE, na="", append = TRUE,sep="\t")
}

#====================================
# Function: read.basefile
#
# Read and parse a 2ch BASEfile
# with position, reporter, reporterID, Int1, Int2, flags, bg1, bg2
#====================================

read.basefile<-function(fname)
{
infile<-readLines(fname)		#Read file
cat("", file="tmp.txt")			#Clear tmp.txt
unlink("tmp.txt")			#remove old file


if (infile[1]=="BASEfile")
{
	currentsection="BASEfile"
	print ("BASEfile OK")
	max.par<-1;
	tablefound<-FALSE;
	line<-1
	
	while (line<=length(infile) && tablefound==FALSE)
	{

		if (infile[line]!="")
		{
			this.line<-unlist(strsplit(infile[line],"\t"))

			if(this.line[1]=="section")
			{
				currentsection<-this.line[2]
				header<-1;
			}
			else if (this.line[1]=="%")	#start of data
			{
				header<-0;
				if (currentsection=="spots")
				{
					tablefound<-TRUE
					data.line<-line
				}
			}
			else if (tablefound==FALSE)
			{
				if (currentsection=="BASEfile")	#Read parameter list, 1=parameter, 2=value
				{
					tmp.cmd<-paste(this.line[1],"<<-\"",this.line[2],"\"", sep="")
					eval(parse(text=tmp.cmd))
#					print(tmp.cmd)
				}
				else if (currentsection=="assays")	#Read assay data
				{
					if (header>=1)			#Read Header info
					{
						if (this.line[1]=="count")	#start of data
						{
							count=as.integer(this.line[2]);
						}
						else
						{
							tmp.cmd<-paste(this.line[1],"<-c(",sep="")
							if (length(this.line)>2)
							{
								for(ii in 2:(length(this.line)-1))
								{
									tmp.cmd<-paste(tmp.cmd,"\"",this.line[ii],"\",", sep="")
								}
							}
							tmp.cmd<-paste(tmp.cmd,"\"",this.line[length(this.line)],"\")", sep="")
							eval(parse(text=tmp.cmd))

						}
					}
					else		#Read Data
					{
						#extract all lines to file until blank line.
						#Substitute #-signs to ~ for R compatibility
						for(ii in 1:nchar(infile[line]))
						{
							if(substr(infile[line],ii,ii)=="#")
								substring(infile[line],ii)<-"~"
						}

						cat(infile[line], file="tmp.txt", sep="\n", append=TRUE)
					}
				}
				else if (currentsection=="spots")		#Read spot info
				{
					if (header==1)				#Read Header info
					{
						if (this.line[1]=="channels")	#start of data
						{
							channels=as.integer(this.line[2]);
						}
						else				#Extract all data and write to temp file
						{
							tmp.cmd<-paste(this.line[1],"<-c(",sep="")

							for(ii in 2:(length(this.line)-1))
							{
							tmp.cmd<-paste(tmp.cmd,"\"",this.line[ii],"\",", sep="")
							}

							tmp.cmd<-paste(tmp.cmd,"\"",this.line[length(this.line)],"\")", sep="")
							eval(parse(text=tmp.cmd))

						}
					}
				}
				else
				{
					print("Error! Unknown section.")
				}
			}
			else		#Tablefound=TRUE!
			{
				print("Line Length Set!")
				line<-length(infile)-1
			}
		}
		else		#End of section (Blank line)
		{
			#Read data from tmp.txt to corresponding matrix
			if (currentsection=="assays")
			{
				assay.data<-read.table("tmp.txt",sep="\t")
				unlink("tmp.txt")
			}
			currentsection="inter-section"
		}
	line<-line+1
	}
}

#Debug

print("Extracting spots")

#Extract all data from SPOTS section
cat(infile[data.line+1:(length(infile)-data.line)], file="tmp.txt", sep="\n")
spots.data<-read.delim("tmp.txt",sep="\t",header=FALSE)
unlink("tmp.txt")

#So, now put everything in the correct format with correct labels
nhybs <- (dim(spots.data)[2] - 3)/(channels*2+1)
int1<-spots.data[(0:(nhybs-1))*5+4]
int2<-spots.data[(0:(nhybs-1))*5+5]
flags.in<-spots.data[(0:(nhybs-1))*5+6]
bg1<-spots.data[(0:(nhybs-1))*5+7]
bg2<-spots.data[(0:(nhybs-1))*5+8]
parameters<-spots.data[0:3]

#Save spots section for return if normalization
trafo.str<-"BASEfile"
trafo.str[2]<-"section\tspots"
trafo.str[3]<-paste("columns\t",paste(columns,collapse="\t",sep=""),collapse="",sep="")
trafo.str[4]<-paste("channels\t",channels,collapse="",sep="")
trafo.str[5]<-paste("assayFields\t",paste(assayFields,collapse="\t",sep=""),collapse="",sep="")
trafo.str[6]<-paste("assays\t",paste(assays,collapse="\t",sep=""),collapse="",sep="")
trafo.str[7]<-"%"

#Write return list and set column names
RG<-list(R=int1, Rb=bg1, G=int2, Gb=bg2, pars=parameters, trafo=trafo.str, Flags=flags.in)
colnames(RG$R)<-paste("Intensity1.",1:dim(RG$R)[2]*2-1, sep="")
colnames(RG$G)<-paste("Intensity2.",1:dim(RG$G)[2]*2, sep="")
colnames(RG$Rb)<-paste("Background1.",1:dim(RG$Rb)[2]*2-1, sep="")
colnames(RG$Gb)<-paste("Background2.",1:dim(RG$Gb)[2]*2, sep="")
colnames(RG$Flags)<-paste("Array",1:dim(RG$Gb)[2], sep="")
colnames(RG$pars)<-columns[0:3]

return(RG)
}

#====================================
# Function: read.basefile.short
#
# Read and parse a 2ch BASEfile
# with position, reporter, reporterID, Int1, Int2
#====================================

read.basefile.short<-function(fname)
{
infile<-readLines(fname)		#Read file
cat("", file="tmp.txt")			#Clear tmp.txt
unlink("tmp.txt")			#remove old file


if (infile[1]=="BASEfile")
{
	currentsection="BASEfile"
	print ("BASEfile OK")
	max.par<-1;
	tablefound<-FALSE;
	line<-1
	
	while (line<=length(infile) && tablefound==FALSE)
	{

		if (infile[line]!="")
		{
			this.line<-unlist(strsplit(infile[line],"\t"))

			if(this.line[1]=="section")
			{
				currentsection<-this.line[2]
				header<-1;
			}
			else if (this.line[1]=="%")	#start of data
			{
				header<-0;
				if (currentsection=="spots")
				{
					tablefound<-TRUE
					data.line<-line
				}
			}
			else if (tablefound==FALSE)
			{
				if (currentsection=="BASEfile")	#Read parameter list, 1=parameter, 2=value
				{
					tmp.cmd<-paste(this.line[1],"<<-\"",this.line[2],"\"", sep="")
					eval(parse(text=tmp.cmd))
#					print(tmp.cmd)
				}
				else if (currentsection=="assays")	#Read assay data
				{
					if (header>=1)			#Read Header info
					{
						if (this.line[1]=="count")	#start of data
						{
							count=as.integer(this.line[2]);
						}
						else
						{
							tmp.cmd<-paste(this.line[1],"<-c(",sep="")
							if (length(this.line)>2)
							{
								for(ii in 2:(length(this.line)-1))
								{
									tmp.cmd<-paste(tmp.cmd,"\"",this.line[ii],"\",", sep="")
								}
							}
							tmp.cmd<-paste(tmp.cmd,"\"",this.line[length(this.line)],"\")", sep="")
							eval(parse(text=tmp.cmd))

						}
					}
					else		#Read Data
					{
						#extract all lines to file until blank line.
						#Substitute #-signs to ~ for R compatibility
						for(ii in 1:nchar(infile[line]))
						{
							if(substr(infile[line],ii,ii)=="#")
								substring(infile[line],ii)<-"~"
						}

						cat(infile[line], file="tmp.txt", sep="\n", append=TRUE)
					}
				}
				else if (currentsection=="spots")		#Read spot info
				{
					if (header==1)				#Read Header info
					{
						if (this.line[1]=="channels")	#start of data
						{
							channels=as.integer(this.line[2]);
						}
						else				#Extract all data and write to temp file
						{
							tmp.cmd<-paste(this.line[1],"<-c(",sep="")

							for(ii in 2:(length(this.line)-1))
							{
							tmp.cmd<-paste(tmp.cmd,"\"",this.line[ii],"\",", sep="")
							}

							tmp.cmd<-paste(tmp.cmd,"\"",this.line[length(this.line)],"\")", sep="")
							eval(parse(text=tmp.cmd))

						}
					}
				}
				else
				{
					print("Error! Unknown section.")
				}
			}
			else		#Tablefound=TRUE!
			{
				print("Line Length Set!")
				line<-length(infile)-1
			}
		}
		else		#End of section (Blank line)
		{
			#Read data from tmp.txt to corresponding matrix
			if (currentsection=="assays")
			{
				assay.data<-read.table("tmp.txt",sep="\t")
				unlink("tmp.txt")
			}
			currentsection="inter-section"
		}
	line<-line+1
	}
}

#Debug

print("Extracting spots")

#Extract all data from SPOTS section
cat(infile[data.line+1:(length(infile)-data.line)], file="tmp.txt", sep="\n")
spots.data<-read.delim("tmp.txt",sep="\t",header=FALSE)
unlink("tmp.txt")

#So, now put everything in the correct format with correct labels
nhybs <- (dim(spots.data)[2] - 2)/channels
int1<-spots.data[(0:(nhybs-1))*2+4]
int2<-spots.data[(0:(nhybs-1))*2+5]
parameters<-spots.data[0:3]

#Save spots section for return if normalization
trafo.str<-"BASEfile"
trafo.str[2]<-"section\tspots"
trafo.str[3]<-paste("columns\t",paste(columns,collapse="\t",sep=""),collapse="",sep="")
trafo.str[4]<-paste("channels\t",channels,collapse="",sep="")
trafo.str[5]<-paste("assayFields\t",paste(assayFields,collapse="\t",sep=""),collapse="",sep="")
trafo.str[6]<-paste("assays\t",paste(assays,collapse="\t",sep=""),collapse="",sep="")
trafo.str[7]<-"%"

#Write return list and set column names
RG<-list(R=int1, Rb=NULL, G=int2, Gb=NULL, pars=parameters, trafo=trafo.str)
colnames(RG$R)<-paste("Intensity1.",1:dim(RG$R)[2]*2-1, sep="")
colnames(RG$G)<-paste("Intensity2.",1:dim(RG$G)[2]*2, sep="")
colnames(RG$pars)<-columns[0:3]

return(RG)
}


#====================================
# Function: read.basefile.short.names
#
# Read and parse a 2ch BASEfile
# with position, reporter, reporterID, geneName, Int1, Int2
#====================================

read.basefile.short.names<-function(fname)
{
infile<-readLines(fname)		#Read file
cat("", file="tmp.txt")			#Clear tmp.txt
unlink("tmp.txt")			#remove old file

if (infile[1]=="BASEfile")
{
	currentsection="BASEfile"
	print ("BASEfile OK")
	max.par<-1;
	tablefound<-FALSE;
	line<-1
	
	while (line<=length(infile) && tablefound==FALSE)
	{

		if (infile[line]!="")
		{
			this.line<-unlist(strsplit(infile[line],"\t"))

			if(this.line[1]=="section")
			{
				currentsection<-this.line[2]
				header<-1;
			}
			else if (this.line[1]=="%")	#start of data
			{
				header<-0;
				if (currentsection=="spots")
				{
					tablefound<-TRUE
					data.line<-line
				}
			}
			else if (tablefound==FALSE)
			{
				if (currentsection=="BASEfile")	#Read parameter list, 1=parameter, 2=value
				{
					tmp.cmd<-paste(this.line[1],"<<-\"",this.line[2],"\"", sep="")
					eval(parse(text=tmp.cmd))
#					print(tmp.cmd)
				}
				else if (currentsection=="assays")	#Read assay data
				{
					if (header>=1)			#Read Header info
					{
						if (this.line[1]=="count")	#start of data
						{
							count=as.integer(this.line[2]);
						}
						else
						{
							tmp.cmd<-paste(this.line[1],"<-c(",sep="")
							if (length(this.line)>2)
							{
								for(ii in 2:(length(this.line)-1))
								{
									tmp.cmd<-paste(tmp.cmd,"\"",this.line[ii],"\",", sep="")
								}
							}
							tmp.cmd<-paste(tmp.cmd,"\"",this.line[length(this.line)],"\")", sep="")
							eval(parse(text=tmp.cmd))

						}
					}
					else		#Read Data
					{
						#extract all lines to file until blank line.
						#Substitute #-signs to ~ for R compatibility
						for(ii in 1:nchar(infile[line]))
						{
							if(substr(infile[line],ii,ii)=="#")
								substring(infile[line],ii)<-"~"
						}

						cat(infile[line], file="tmp.txt", sep="\n", append=TRUE)
					}
				}
				else if (currentsection=="spots")		#Read spot info
				{
					if (header==1)				#Read Header info
					{
						if (this.line[1]=="channels")	#start of data
						{
							channels=as.integer(this.line[2]);
						}
						else				#Extract all data and write to temp file
						{
							tmp.cmd<-paste(this.line[1],"<-c(",sep="")

							for(ii in 2:(length(this.line)-1))
							{
							tmp.cmd<-paste(tmp.cmd,"\"",this.line[ii],"\",", sep="")
							}

							tmp.cmd<-paste(tmp.cmd,"\"",this.line[length(this.line)],"\")", sep="")
							eval(parse(text=tmp.cmd))

						}
					}
				}
				else
				{
					print("Error! Unknown section.")
				}
			}
			else		#Tablefound=TRUE!
			{
				print("Line Length Set!")
				line<-length(infile)-1
			}
		}
		else		#End of section (Blank line)
		{
			#Read data from tmp.txt to corresponding matrix
			if (currentsection=="assays")
			{
				assay.data<-read.table("tmp.txt",sep="\t")
				unlink("tmp.txt")
			}
			currentsection="inter-section"
		}
	line<-line+1
	}
}

#Debug

print("Extracting spots")

#Extract all data from SPOTS section
cat(infile[data.line+1:(length(infile)-data.line)], file="tmp.txt", sep="\n")
spots.data<-read.delim("tmp.txt",sep="\t",header=FALSE)
unlink("tmp.txt")

#So, now put everything in the correct format with correct labels
nhybs <- (dim(spots.data)[2] - 4)/channels
int1<-spots.data[(0:(nhybs-1))*2+5]
int2<-spots.data[(0:(nhybs-1))*2+6]
parameters<-spots.data[0:4]

#Save spots section for return if normalization
trafo.str<-"BASEfile"
trafo.str[2]<-"section\tspots"
trafo.str[3]<-paste("columns\t",paste(columns,collapse="\t",sep=""),collapse="",sep="")
trafo.str[4]<-paste("channels\t",channels,collapse="",sep="")
trafo.str[5]<-paste("assayFields\t",paste(assayFields,collapse="\t",sep=""),collapse="",sep="")
trafo.str[6]<-paste("assays\t",paste(assays,collapse="\t",sep=""),collapse="",sep="")
trafo.str[7]<-"%"

#Write return list and set column names
RG<-list(R=int1, Rb=NULL, G=int2, Gb=NULL, pars=parameters, trafo=trafo.str)
colnames(RG$R)<-paste("Intensity1.",1:dim(RG$R)[2]*2-1, sep="")
colnames(RG$G)<-paste("Intensity2.",1:dim(RG$G)[2]*2, sep="")
colnames(RG$pars)<-columns[0:4]

return(RG)
}
