#====================================
# Plugin: R: Normalization Between Arrays (Scaling)
# Date: 2003/09/17
# v: 0.92
#
# Normalization between arrays using R functions from Bioconductor(LIMMA) and SMA.
#
# By: Ola Spjuth, ola@crossoverbio.com
# for The Linnaeus Centre for Bioinformatics, www.lcb.uu.se
#====================================

#Include functions for BASEfile read/write
source("/usr/local/base/plugins/R/parse.basefile.R")

#Disable warnings since they tend to scare some users :-(
options(warn=-1)

#Load required libraries
library(sma)
library(limma)

#Read data from stdin
RG<-read.basefile.short("stdin.txt")
print("Data loaded")

RG.trafo<-list(R=RG$R,G=RG$G)
MAraw <- MA.RG(RG.trafo)			#Convert to MA
no.assays<-dim(RG.trafo$R)[2]			#Number of Assays

#Normalize
if (length(try(MA <- normalizeBetweenArrays(MAraw),TRUE))==1)
{
	print("Error in normalization between arrays.")
	stop("Normalization terminated.")	
}

#Inverse transform back to RG from MA
RG$R=round(2^(2*MA$A+MA$M))^(0.5)
RG$G=round(2^(2*MA$A-MA$M))^(0.5)

#Write output as transformation
write.transformation("stdout.txt",RG)

#Construct images
	bitmap(file="boxplot1.png", type = "png16m", height = 10, width = 10, res = 72)
		boxplot(MAraw$M~col(MAraw$M),col=rainbow(no.assays),names=paste("array",1:no.assays,sep=""),main="Before Normalization")
		abline(0,0,col="blue")
	dev.off()

	bitmap(file="boxplot2.png", type = "png16m", height = 10, width = 10, res = 72)
		boxplot(MA$M~col(MA$M),col=rainbow(no.assays),names=paste("array",1:no.assays,sep=""),main="After Normalization")
		abline(0,0,col="blue")
	dev.off()

#Result page as HTML
#===================
cat("<html><body>", file="Results.html")
cat(paste("<h1>Results from normalization using method: ",eval(parse(text=paste("norm.methods$",norm.method,sep=""))),"</h1><p>",sep=""), file="Results.html", append=TRUE)

cat("<img src=\"boxplot1.png\"><p>", file="Results.html", append=TRUE)
cat("<img src=\"boxplot2.png\"><p>", file="Results.html", append=TRUE)

cat("<p><hr><p>", file="Results.html", append=TRUE)
cat("Normalization and Plots were obtained using R-libraries <a href=\"http://bioinf.wehi.edu.au/limma/\">Bioconductor(LIMMA)</a> and <a href=\"http://stat-www.berkeley.edu/users/terry/zarray/Software/smacode.html\">SMA</a>. <p>", file="Results.html", append=TRUE)
cat("(c) 2003 Ola Spjuth, <a href=\"http://www.crossoverbio.com\">www.crossoverbio.com <p>", file="Results.html", append=TRUE)

cat("</body></html>", file="Results.html", append=TRUE)

print("Finished")
